//=============================================================================
// AutoNavPawn.
//=============================================================================
class AutoNavPawn expands ScriptedPawn;

// Exploring
var(Exploring)	class<Breadcrumb>	PathType;
var(Exploring)	int					ExplorationWill;
var(Exploring)	float				ExplorationRadius;
var				BreadcrumbPath		CurrentPath;
var				Breadcrumb			NavigationTarget;
var				Actor				NavigationReason;
var				Vector				NavigationDest;
var				bool				bHasCrumb;
var(Exploring)	bool				bFindFood; // explores for food

// Eating
var			Actor					Food; // Mmmm wood! Wait, what?!?
var(Eating)	class<Actor>			FoodType; // Ok, now I won't eat wood again---- mmmm, bioglobs! Hey Slith, process them for me! ^_^
var(Eating)	float					EatRate, EatScale, EatInterval; // Ok, eating slow and always-- aw, mapper made me to go fast as a bolt. :< Not gonna feel the taste... Ew! He was right.
var(Eating)	float					EatHeal; // Everything healthy for my strong and mushy stomach!
var			float					FoodSize; // Mmmmm, so much spaghetti! Who wanna serve for meatball? *looks around*
var			float					CurrentFoodSize;
var(Eating)	float					SwallowSize; // *gulp* Oh hi there! I can hear you from inside!
var(Eating)	float					EatDelay; // Waiting for the waiter... he does not looks bad!

// Moving
var(Steps)	float					StepSize, StepHeight; // I'm a stridicus and my slime can't be mushed!
var(Steps)	float					JumpSize; // Hoof!
var			vector					StepBuffer; // I have memory to know where I am going! Er... but not where I am! *hugs you*

function bool GapBetween(vector Into, vector From, optional float DetectionRange, optional float MaxGapHeight)
{
	local vector v, w, n;

	if ( MaxGapHeight <= 0 )
		MaxGapHeight = CollisionHeight;

	if ( DetectionRange <= 0 )
		DetectionRange = 2;

	n = Normal(Into - From);
	for ( v = Into; VSize(From - v) > 1; v += n * (VSize(Into - From) / DetectionRange) )
	{
		w = v;
		w.Z -= MaxGapHeight;

		if ( !FastTrace(w, v) )
			return true;
	}

	return false;
}

function StepToward(Actor Other)
{
	StepTo(Other.Location);
}

function StepTo(optional Vector Dest, optional bool bUseBuffer)
{
	if ( bUseBuffer )
		Dest = StepBuffer;

	else
		StepBuffer = Dest;

	Log(self@"moving towards"@Dest);

	Velocity += Normal(Dest - Location) * StepSize;
	Velocity.Z -= (Normal(Dest - Location) * StepSize).Z * StepHeight;

	if ( GapBetween(Dest, Location) )
		Velocity.Z += JumpSize;
}

function bool HasActor(class<Actor> Type)
{
	local Actor A;

	foreach AllActors(Type, A)
		return true;

	return false;
}

function Actor ClosestTagObject(name Tag, vector Loc, optional class<Actor> Type)
{
	local Actor A;
	local Actor Closest;

	if ( Type == None )
		Type = class'Actor';

	foreach AllActors(Type, A, Tag)
		if ( Closest == None || VSize(A.Location - Loc) < VSize(Closest.Location - Loc) )
			Closest = A;

	return A;
}

function Actor ClosestActor(class<Actor> Type, Vector Loc, optional float Limit)
{
	local Actor A;
	local Actor R;

	if ( Limit <= 0 )
		foreach AllActors(Type, A)
		{
			if ( A != None && ( R == None || VSize(A.Location - Loc) < VSize(R.Location - Loc) ) )
				R = A;
	
			else if ( Limit > 0 )
				Log(VSize(A.Location - Loc)@Limit);
		}

	else
		foreach RadiusActors(Type, A, Limit)
			if ( A != None && ( R == None || VSize(A.Location - Loc) < VSize(R.Location - Loc) ) )
				R = A;
	
			else if ( Limit > 0 )
				Log(VSize(A.Location - Loc));

	return R;
}

function int ActorsAround(class<Actor> Type, float Radius)
{
	local int i;
	local Actor A;

	foreach RadiusActors(Type, A, Radius)
		i++;

	return i;
}

function LayBreadcrumb()
{
	local Vector Loc, Norm, Targ;

	Targ.Z = -256;
	Trace(Loc, Norm, Location + Targ);

	Spawn(PathType, self,, Loc);
}

function bool KnowsArea()
{
	return ExplorationWill > 0 && ActorsAround(PathType, ExplorationRadius) > ExplorationWill;
}

function LookAround();

state Exploring
{
	function vector GetWanderDest(optional bool bCountBreadcrumbs)
	{
		local vector v, n, r;
		local vector Results[25];
		local int i, j;

		v.Z = -0.1;

		for ( v.X = -1; v.X <= 1; v.X += 0.5 )
			for ( v.Y = -1; v.Y <= 1; v.Y += 0.5 )
			{
				Trace(Results[i], n, Location + (v * 512));

				i++;
			}

		j = 0;
		for ( r = Results[Rand(25)]; bCountBreadcrumbs && Breadcrumb(ClosestActor(PathType, r, 192)) == None && j <= 25; r = Results[Rand(25)] )
			j++;

		return r;
	}

Begin:
	if ( OrderTag != 'None' )
	{
		if ( NavigationReason == None && Food == None )
		{
			NavigationReason = ClosestTagObject(OrderTag, Location);

			if ( NavigationReason == None )
			{
				OrderTag = 'None';

				GoTo('Begin');
			}
		}

		if ( Food == None )
			NavigationDest = NavigationReason.Location;

		else
		{
			Log(self@"is going for food:"@Food);

			NavigationDest = Food.Location;
		}
	}

	else
		NavigationDest = GetWanderDest(true);

	if ( ClosestActor(PathType, NavigationDest, 48) == None )
		NavigationTarget = PathType.static.CreateBreadcrumb(self, NavigationDest);

	else
		NavigationTarget = Breadcrumb(ClosestActor(PathType, NavigationDest));

FindMoveTarget:
	if ( NavigationTarget != None )
	{
		if ( Breadcrumb(ClosestActor(PathType, Location)) == None || Breadcrumb(ClosestActor(PathType, Location)).FindPathToBreadcrumb(NavigationTarget) == None )
		{
			Log(self@"moving towards"@NavigationTarget);

			StepToward(NavigationTarget);
		}

		else
		{
			Log(self@"is following"@ClosestActor(PathType, Location)@"to"@Breadcrumb(ClosestActor(PathType, Location)).FindPathToBreadcrumb(NavigationTarget).Destination);

			StepToward(Breadcrumb(ClosestActor(PathType, Location)).FindPathToBreadcrumb(NavigationTarget).Destination);
		}
	}

	else if ( FastTrace(NavigationDest) )
		StepTo(NavigationDest);

ExMoving:
	while ( VSize(Location - NavigationTarget.Location) > 48 )
	{
		Sleep(0.6);
		StepTo(, true);

		Food = ClosestActor(FoodType, Location, 128);

		if ( Food != None )
		{
			if ( VSize(Food.Location - Location) < 32)
			{
				GoToState('Eating');

					
			}

			OrderTag = Food.Tag;
		}

		else
			GoTo('Begin');
	}

	if ( !KnowsArea() || Food != None )
		GoTo('Begin');

	GoTo('Explored');

Explored:
	LookAround();

	GoToState('Wandering');
}

state Eating
{
	Begin:
		FoodSize = Food.Mass * Food.DrawScale;

	Eat:
		CurrentFoodSize = FoodSize;

		Sleep(EatDelay);

		if ( CurrentFoodSize < SwallowSize )
			GoTo('Swallow');

	Chew:
		Food.Mass -= EatRate;
		Food.DrawScale -= EatScale;
		Health += EatHeal;

		Sleep(EatInterval);

		GoTo('Eat');

	Swallow:
		Food.Destroy();

		GoToState('Exploring');		
}

defaultproperties
{
     PathType=Class'GusPackII.Breadcrumb'
     ExplorationRadius=1024.000000
     FoodType=Class'Engine.Carcass'
     EatRate=2.000000
     EatScale=0.210000
     EatInterval=0.800000
     EatHeal=9.000000
     SwallowSize=2.500000
     EatDelay=1.000000
     StepSize=48.000000
     StepHeight=0.400000
     JumpSize=24.000000
     Orders=Exploring
     RefireRate=8.000000
     bIsWuss=True
     RangedProjectile=Class'GusPackII.BioToxin'
     Health=80
     AttitudeToPlayer=ATTITUDE_Ignore
     Intelligence=BRAINS_HUMAN
     Skill=2.000000
     Sprite=Texture'Engine.S_Flag'
     DrawScale=2.000000
}
