//=============================================================================
// BotCannon.
//=============================================================================
class BotCannon expands Projectile;

var()	float			GunOffset;
var()	class<Weapon>	MyGunType;
var		Weapon			MyGun;
var		Pawn			Target;
var		Pawn			User;
var()	float			TargetingRadius;
var()	float			Health;
var()	Sound			DestroySound;
var()	float			MaxWallShallowness;
var		AttachInfo		GunAttach;
var		AttachInfo		MyAttach;

function TakeDamage(int Damage, Pawn EventInstigator, vector HitLocation, vector Momentum, name DamageType)
{
	Health -= Damage;

	if ( Health <= 0 )
		ExplodeArrow();

	Velocity += Momentum / Mass;
}

simulated function ExplodeArrow()
{
	MyGun.Destroy();
	PlaySound(DestroySound);

	Destroy();
}

simulated event Timer()
{
	local Pawn p;

	if ( Target == None )
		foreach RadiusActors(class'Pawn', p, TargetingRadius)
			if ( CheckSeekable(p, User) )
			{
				Target = p;

				return;
			}

	else if ( MyGun != None )
	{
		MyGun.AmmoType = None;
		MyGun.Fire(1.0);
	}
}

simulated function Tick(float TimeDelta)
{
	local Rotator NewGunRot;

	GunAttach.AttachOffset = Vector(Rotation) * GunOffset;

	if ( Target != None  )
		if ( VSize(Location - Target.Location) > TargetingRadius )
		{
			Target = None;

			return;
		}

		else if ( MyGun != None )
			MyGun.SetRotation(Rotator(Target.Location - Location));

	if ( MyGun == None )
	{
		NewGunRot = Rotation;
		NewGunRot.Pitch += 16384;	
		NewGunRot.Yaw = FRand() * 65536;

		MyGun = Spawn(MyGunType, self,, Location + (Vector(Rotation) * GunOffset), NewGunRot);
		MyGun.SetPhysics(PHYS_None);
		MyGun.bRotatingPickup = false;
		MyGun.GoToState('Active');

		GunAttach = Spawn(class'AttachInfo', self);
		GunAttach.Other = self;
		GunAttach.Attached = MyGun;
		GunAttach.AttachOffset = Vector(Rotation) * GunOffset;
	}
}

simulated function BeginPlay()
{
	if ( Owner == None )
		return;

	if ( Pawn(Owner) != None )
		User = Pawn(Owner);

	else
		User = Pawn(Owner.Owner);

	if ( User == None )
		return;
}

simulated function Touch(Actor Other)
{
	local Pawn P;

	if ( Pawn(Other) != None && CheckSeekable(Pawn(Other), User) )
	{
		P = Pawn(Other);

		P.Health -= Damage;

		if ( P.Health <= 0 )
			P.KilledBy(User);

		ExplodeArrow();

		return;
	}

	else
	{
		MyAttach = Spawn(class'AttachInfo', self);
		MyAttach.Other = Other;
		MyAttach.Attached = self;
		MyAttach.AttachOffset = Other.Location - Location;

		SetPhysics(PHYS_None);
	}
}

simulated event bool MonsterFriendly(ScriptedPawn Other)
{
	return Other != None && Other.AttitudeTo(User) == ATTITUDE_Friendly;
}

simulated event bool CheckSeekable(Pawn Other, Pawn FireAgent)
{
	return (
		Other != None
		&&
		(
			FireAgent == None
			^^
			(
				FastTrace(Other.Location)
				&& Other != FireAgent
				&& !Other.IsInState('Dying')
				&&
				(
					!MonsterFriendly(ScriptedPawn(Other))
					||
					(
						!Level.Game.bTeamGame
						|| Other.PlayerReplicationInfo == None
						|| FireAgent.PlayerReplicationInfo == None
						|| Other.PlayerReplicationInfo.Team != FireAgent.PlayerReplicationInfo.Team
					)
				)
			)
		)
	);
}

simulated function PostBeginPlay()
{
	local Rotator NewGunRot;

	NewGunRot = Rotation;
	NewGunRot.Pitch += 16384;	
	NewGunRot.Yaw = FRand() * 65536;

	MyGun = Spawn(MyGunType, self,, Location + (Vector(Rotation) * GunOffset), NewGunRot);
	MyGun.bRotatingPickup = false;
	MyGun.GoToState('Active');
	MyGun.SetPhysics(PHYS_None);

	SetPhysics(PHYS_Projectile);

	SetTimer(0.1, true);

	GunAttach = Spawn(class'AttachInfo', self);
	GunAttach.Other = self;
	GunAttach.Attached = MyGun;
	GunAttach.AttachOffset = Vector(Rotation) * GunOffset;

	Velocity = Vector(Rotation) * speed;
}

simulated function HitWall(Vector Normal, Actor Wall)
{
	SetPhysics(PHYS_None);

	if ( Normal dot Vector(Rotation) > MaxWallShallowness )
		ExplodeArrow();

	else
		SetRotation(Rotator(-Normal));
}

simulated function Landed(Vector Normal)
{
	HitWall(Normal, None);
}

defaultproperties
{
     GunOffset=-30.000000
     MyGunType=Class'Botpack.ripper'
     TargetingRadius=256.000000
     Health=35.000000
     DestroySound=Sound'UnrealShare.Stinger.StingerAltFire'
     MaxWallShallowness=0.400000
     speed=1250.000000
     Damage=20.000000
     SpawnSound=Sound'UnrealShare.Razorjack.BladeHit'
     Mesh=LodMesh'UnrealShare.ArrowM'
     CollisionRadius=24.000000
     CollisionHeight=8.000000
     Mass=15.000000
}
