//=============================================================================
// Breadcrumb.
//=============================================================================
class Breadcrumb expands Actor;

var		BreadcrumbPath	Pathes;
var		Breadcrumb		InternalNext, InternalPrev, InternalFirst;
var()	float			MaxBreadcrumbDistance, WalkHeight;
var		vector			OldLocation;
var		bool			bFindingPath;
var		NavigationPoint	Correspond;

enum PathType
{
	PT_None,
	PT_NavigationPoints,
	PT_Breadcrumbs
};

static function Breadcrumb CreateBreadcrumb(Actor Anchor, vector NewLocation, optional NavigationPoint Correspondent, optional Breadcrumb ForcedPathes[64])
{
	local Breadcrumb B;
	local int i;

	B = Anchor.Spawn(default.class, Anchor,, NewLocation);
	B.Correspond = Correspondent;

	for ( i = 0; ForcedPathes[i] != None && i < 64; i++ )
	{
		B.ForceConnection(ForcedPathes[i]);

		ForcedPathes[i].ForceConnection(B);
	}

	return B;
}

static function ConnectBreadcrumbs(Breadcrumb A, Breadcrumb B)
{
	A.ForceConnection(B);
	B.ForceConnection(A);
}

function BreadcrumbPath AppendPathes(BreadcrumbPath A, BreadcrumbPath B, optional bool bTemp)
{
	if ( A == None )
		return B;

	if ( B == None )
		return A;

	A.AddPath(B, bTemp);
	return A;
}

function SetUp()
{
	local Breadcrumb O;
	local LiftExit N;

	if ( LiftCenter(Correspond) != None )
		foreach AllActors(class'LiftExit', N, LiftCenter(Correspond).LiftTag)
			for ( O = InternalFirst; O != None; O = O.InternalNext )
				if ( O.Correspond == N )
				{
					ForceConnection(O);
					O.ForceConnection(self);
	
					break;
				}
}

function UpdatePathes()
{	
	local int i;

	local Breadcrumb BC;
	local Breadcrumbpath NewPath;
	local Breadcrumbpath Current;

	for ( BC = InternalFirst; BC != None; BC = BC.InternalNext )
	{
		if ( BC == InternalPrev || BC == self )
			break;

		if ( BC != None && VSize(BC.Location - Self.Location) <= MaxBreadcrumbDistance && BreadcrumbReachable(BC) && BC != self && ( Pathes == None || Pathes.SegmentWithBreadcrumb(BC) == None ) )
		{
			NewPath = Spawn(class'BreadcrumbPath', self);

			NewPath.Destination = BC;

			if ( Pathes != None )
				Pathes.AddPath(NewPath);

			else
				Pathes = NewPath;

			i++;
		}
	}
}

function Tick(float TimeDelta)
{
	if ( Location != OldLocation )
	{
		if ( Pathes != None )
			Pathes.DeletePath();

		UpdatePathes();
	}

	OldLocation = Location;
}

function ForceConnection(Breadcrumb Other)
{
	local BreadcrumbPath P;

	P = Spawn(class'BreadcrumbPath', self);
	P.Destination = Other;
	
	Pathes.AddPath(P);
}

function bool BreadcrumbReachable(Breadcrumb Other)
{
	local bool r;
	local Vector v;
	local Vector w;
	local Vector n;

	r = FastTrace(Other.Location) && -1024 < Other.Location.Z - Location.Z && Other.Location.Z - Location.Z < 256 && VSize(Other.Location - Location) < MaxBreadcrumbDistance;

	n = Other.Location - Location;

	for ( v = Location; v != Other.Location && VSize(Other.Location - v) > 1; v += n / 3 )
	{
		w = v;
		w.Z -= WalkHeight * abs(v.z);

		r = r && !FastTrace(v, w);
	}

	return r;
}

function BreadcrumbPath FindPathToBreadcrumb(Breadcrumb Destination, optional BreadcrumbPath Path)
{
	local BreadcrumbPath Current;
	local BreadcrumbPath NewPath;
	local BreadcrumbPath PossiblePathes;
	local BreadcrumbPath BP;
	local BreadcrumbPath Closest;

	if ( IsInState('Deactivated') || Destination == None )
		return None;

	for ( Current = Pathes; Current != None; Current = Current.Next )
	{
		if ( Current.Destination == Destination )
			return AppendPathes(Path, Current);
		
		else 
			if ( !BreadcrumbInPathes(Current.Destination, Path) )
			{
				if ( PossiblePathes != None )
					PossiblePathes.AddOpt(Current.Destination.FindPathToBreadcrumb(Destination, AppendPathes(Path, Current)));

				else
					PossiblePathes = Current.Destination.FindPathToBreadcrumb(Destination, AppendPathes(Path, Current));
			}
	}

	for ( BP = PossiblePathes; BP != None; BP = BP.Opt )
		if ( Closest == None || BP.PathSize() < Closest.PathSize() )
			Closest = BP;

	return Closest;
}

function ProccessList()
{	
	local Breadcrumb C, U;

	InternalPrev  = None;
	InternalNext  = None;
	InternalFirst = None;

	foreach AllActors(class'Breadcrumb', C)
	{
		if ( C.InternalNext == None && C != self )
		{
			C.InternalNext = Self;
			InternalPrev = C;
		}
	}

	if ( InternalPrev != None )
	{
		for ( U = InternalPrev; U.InternalPrev != None; U = U.InternalPrev )
			continue;

		InternalFirst = U;	

		return;
	}

	InternalFirst = self;
}

function PreBeginPlay()
{
	if ( OldLocation == Location )
		OldLocation += vect(1,0,0); // security fix for startup path generation in case location is 0,0,0

	ProccessList();
	UpdatePathes();
}

function float BreadcrumbCost(Pawn Other)
{
	return 5;
}

function bool BreadcrumbInPathes(Breadcrumb Current, BreadcrumbPath Pathes)
{
	local BreadcrumbPath BP;

	for ( BP = Pathes; BP != None; BP = BP.Next )
		if ( BP.Destination == Current )
			return true;

	return false;
}

function BreadcrumbPath PathAt(BreadcrumbPath Root, int Index)
{
	local int i;
	local BreadcrumbPath BP;

	for ( BP = Root; BP.Next != None; BP = BP.Next )
	{
		if ( i == Index )
			return BP;

		i++;
	}

	return none;
}

function Leaving(Pawn Other, BreadcrumbPath Next, Breadcrumb Destination)
{
	Next.ControlEntrance(Other, Destination);

	Destination.Entering(Other, Next, Destination);
}

function Entering(Pawn Other, BreadcrumbPath Origin, Breadcrumb Destination);

auto	state()	Activated	{}
		state()	Deactivated	{}

defaultproperties
{
     MaxBreadcrumbDistance=1250.000000
     WalkHeight=128.000000
     Sprite=Texture'Engine.S_Actor'
     DrawScale=0.250000
     CollisionHeight=40.000000
     bCollideActors=True
}
