//=============================================================================
// HotChunk.
//=============================================================================
class HotChunk expands UTChunk;

// Propagator properties
var()	class<Actor>	FirePropagator;
var()	float			PropagatorSpeedFactor;

// Cooling down properties
var()	float			CoolDownTime;
var()	float			CoolDownFactor;
var()	bool			bCoolDown;

// Mesh effect properties
var()	class<Effects>	MeshEffectType;
var()	float			EffectDrawScale;
var()	bool			bCopyMeshToEffect, bChangeMeshEffectTexture;
var()	Texture			MeshEffectTexture;
var		Effects			MeshEffect;

simulated function

simulated function HitWall(Vector HitLocation, Actor HitWall)
{
	Explode(HitLocation, Normal(HitLocation - Location));

	Destroy();
}

simulated function Explode(vector HitLocation, vector HitNormal)
{
	local Actor A;
	
	Super.Explode(HitLocation, HitNormal);
	
	A = Spawn(FirePropagator, self, '');
	A.Velocity = VRand() * PropagatorSpeedFactor;
}

simulated function Tick(float TimeDelta)
{
	MeshEffect.Velocity += (MeshEffect.Location - Location) * TimeDelta;
}

auto state() Hot
{
Begin:
	MeshEffect = Spawn(MeshEffectType);

	if ( bCopyMeshToEffect )
	{
		MeshEffect.Mesh = Mesh;

		MeshEffect.DrawScale *= EffectDrawScale;
	}

	if ( bChangeMeshEffectTexture )
	{
		MeshEffect.Skin = MeshEffectTexture;
		MeshEffect.Texture = MeshEffectTexture;
	}

	if ( bCoolDown )
		GoToState('CoolingDown');
}

state CoolingDown
{
	simulated function Tick(float TimeDelta)
	{
		Global.Tick(TimeDelta);

		if ( HotEffect(MeshEffect) != None )
			HotEffect(MeshEffect).CoolDown(TimeDelta, CoolDownTime * CoolDownFactor);
	}

	simulated function ZoneChange(ZoneInfo NewZone)
	{
		if ( NewZone.bWaterZone )
			CoolDownFactor = 2;

		else
			CoolDownFactor = 1;
	}
}

state() Cold
{
	ignores Tick, Explode;
}

defaultproperties
{
     FirePropagator=Class'GusPackII.HotPropagator'
     PropagatorSpeedFactor=24.000000
     CoolDownTime=32.000000
     CoolDownFactor=1.000000
     bCoolDown=True
     MeshEffectType=Class'GusPackII.HotEffect'
     EffectDrawScale=1.700000
     bCopyMeshToEffect=True
     bChangeMeshEffectTexture=True
     InitialState=Hot
     Skin=Texture'Botpack.ChunkGlow.Chunk_a00'
     Mesh=LodMesh'Botpack.chunk3M'
}
