/*
 * If I can't do it in ZDoom,
 * let's do it in UT99 :D
 */
class MushMatch expands TeamGamePlus;


var bool bSelected, bHumanVictory, bStarted, bMatchEnd;
var byte winTeam;
var HateList Hates;
var BeaconList Beacons;

simulated function EndGame(string Reason)
{
	Hates = None;
	Beacons = None;

	Super.EndGame(Reason);
}

function HateList RegisterHate(Pawn Victim, Pawn InstigatedBy)
{
	if ( Hates == None )
	{
		Hates = Spawn(class'GusPackII.HateList', InstigatedBy);
		Hates.Victim = Victim;
		Hates.First = Hates;
	
		// Log("#1#"$Hates@"I("$InstigatedBy$") O("$Victim$")");
		return Hates;
	}
	
	if ( Hates.CheckHate(InstigatedBy, Victim) != None )
		return Hates.CheckHate(InstigatedBy, Victim);
	
	Hates.AddHate(Victim, InstigatedBy);
	// Log("#2#"$Hates.ListHead()@"I("$InstigatedBy$") O("$Victim$")");
	
	return Hates.ListHead();
}

simulated function BeginPlay()
{
	local navigationPoint np;

	Super.BeginPlay();
	
	bMatchEnd = false;
	
	foreach AllActors(class'navigationPoint', np)
		if ( FRand() < 0.05 ) Spawn(class'SporeCanister',,, np.location);
}

simulated function PostBeginPlay()
{
	local Pawn P;

	Super.PostBeginPlay();
	
	for ( P = Level.PawnList; P != none; P = P.nextPawn )
	{
		if ( P.bIsPlayer && p.PlayerReplicationInfo != none )
				p.PlayerReplicationInfo.Team = 0;
	}
}


simulated function ScoreKill(Pawn Killer, Pawn Other)
{
	local byte ok;
	
	local vector hLoc, hMom;
	local int dmg;
	
	hLoc = Other.Location;
	hMom = vect(0,0,0);
	dmg = 32767;
	
	if ( !bSelected )
		Other.PlayerReplicationInfo.Deaths = 0;
	
	if ( Other.PlayerReplicationInfo.Team == 0 && bSelected )
		Broadcastmessage(Other.PlayerReplicationInfo.Playername@"is out of the game! He/she was human!", true, 'CriticalEvent');
		
	else if ( bSelected )
		Broadcastmessage(Other.PlayerReplicationInfo.Playername@"is out of the game! He/she was a mush!", true, 'CriticalEvent');
		
	MushMatchMutator(BaseMutator).MutatorTakeDamage(dmg, Other, Killer, hLoc, hMom, 'death');
	
	CheckEnd();

    if ( !Killer.bIsPlayer || !Other.bIsPlayer || Killer.PlayerReplicationInfo == none || Other.PlayerReplicationInfo == none )
	{
		Super(DeathmatchPlus).ScoreKill(Killer, Other);
		
		return;
	}
	
	ok = Killer.PlayerReplicationInfo.Score;
		
	if ( Killer.PlayerReplicationInfo.Team != Other.PlayerReplicationInfo.Team )
		Killer.PlayerReplicationInfo.Score += 1;
		
	else
		Killer.PlayerReplicationInfo.Score -= 1;

	if ( !bSelected || bMatchEnd )
		return;

	if ( PlayerPawn(Other) != none )
		PlayerPawn(Other).PlayerReplicationInfo.Deaths += 1;
}
	
simulated function CheckEnd()
{
	local bool h, m;
	local Pawn p;
	
	if ( !bSelected || bMatchEnd )
		return;
	
	for ( p = Level.PawnList; p != none; p = p.nextPawn )
		if ( p.bIsPlayer && p.PlayerReplicationInfo != none && !p.IsInState('Dying') && p.PlayerReplicationInfo.Deaths == 0 )
		{
			if ( p.PlayerReplicationInfo.Team == 0 )
				h = true;
		
			if ( p.PlayerReplicationInfo.Team == 1 )
				m = true;
		}
		
	if ( !h || !m )
	{
		if ( h )
			winTeam = 0;
			
		else
			winTeam = 1;
	
		Broadcastmessage("The"@TeamColor[winTeam]@GameEndedMessage);
		SetEndCams("teamstand");
	}
}

simulated function bool SetEndCams(string Reason)
{
	local float EndTime;
	local Pawn P, RWinner;
	local PlayerPawn Player;

	bMatchEnd = true;
	EndTime = Level.TimeSeconds + 3.0;
	GameReplicationInfo.GameEndedComments = "The"@TeamColor[winTeam]@GameEndedMessage;
	log("Game ended at "$EndTime);
	
	for ( P=Level.PawnList; P!=None; P=P.nextPawn )
	{
		if ( P.bIsPlayer && P.PlayerReplicationInfo != none && P.PlayerReplicationInfo.Team == winTeam && P.PlayerReplicationInfo.Deaths <= 0 && ( RWinner == none || P.PlayerReplicationInfo.Score > RWinner.PlayerReplicationInfo.Score ) )
			RWinner = P;
	}
	
	for ( P=Level.PawnList; P!=None; P=P.nextPawn )
	{
		Player = PlayerPawn(P);
		
		if ( Player != None )
		{
			if (!bTutorialGame)
				PlayWinMessage(Player, (Player.PlayerReplicationInfo.Team == winTeam));
				
			Player.bBehindView = true;
			
			if ( RWinner != None )
				Player.ViewTarget = RWinner;
				
			else
				Player.ViewTarget = Player;
				
			Player.ClientGameEnded();
		}

		P.GotoState('GameEnded');
	}
	CalcEndStats();
	return true;
}

simulated function BeaconList StrapBeacon(Pawn Other, optional Pawn Shooter)
{
	if ( Other == None )
		return None;
		
	if ( Beacons == None )
	{
		Beacons = Spawn(class'BeaconList', Other);
		Beacons.Instigator = Shooter;
	
		Broadcastmessage(Shooter.PlayerReplicationInfo.PlayerName@"strapped a suspect beacon on"@Other.PlayerReplicationInfo.PlayerName$"!", true, 'CriticalEvent');
	
		return Beacons;
	}
	
	if ( Beacons.CheckBeacon(Other) )
		return Beacons.FindBeacon(Other);
	
	Beacons.ListHead().Next = Spawn(class'BeaconList', Other);
	Beacons.ListHead().Instigator = Shooter;
	
	Broadcastmessage(Shooter.PlayerReplicationInfo.PlayerName@"strapped a suspect beacon on"@Other.PlayerReplicationInfo.PlayerName$"!", true, 'CriticalEvent');
	
	return Beacons.ListHead();
}

simulated function int NumHates()
{
	if ( Hates == None )
		return 0;
		
	return Hates.ListSize();
}

simulated function byte AssessBotAttitude(Bot aBot, Pawn Other)
{
	local Pawn P;

	if ( Other.PlayerReplicationInfo == none )
		return 2;

	if ( aBot.PlayerReplicationInfo.Team == 0 )
	{
		if (
			Other.bIsPlayer
			&& Other.FindInventoryType(class'GusPackII.NotMush') == None
			&&
			(
				(
					Other.PlayerReplicationInfo != none
					&& Other.PlayerReplicationInfo.Team == 1
					&& Other.FindInventoryType(class'GusPackII.SpottedMush') != none
				)
				||
				(
					Hates != None 
					&& Hates.CheckHate(Other, aBot) != None
				)
				||
				(
					Beacons != None
					&& Beacons.CheckBeacon(Other)
					&& Beacons.FindBeacon(Other).Instigator.FindInventoryType(class'GusPackII.SpottedMush') == None
				)
			)
		)
			return 1;
		
		if ( FRand() < 0.3 )
			return 3;
		
		return 2;
	}
	
	if ( aBot.PlayerReplicationInfo.Team == 1 )
	{
		if ( Other.PlayerReplicationInfo.Team == 0 )
		{
			if ( aBot.FindInventoryType(class'GusPackII.SpottedMush') != None )
				return 1;
		
			if ( Other.CanSee(aBot) )
				return 2;
		
			for ( P = Level.PawnList; P != None; P = P.NextPawn )
				if ( P.bIsPlayer && P.PlayerReplicationInfo != None && P.PlayerReplicationInfo.Team == 0 && P.LineOfSightTo(aBot) && P != Other )
					return 2;
		
			else
			{
				if ( Other.MoveTarget != None && VSize(Other.Location - Other.MoveTarget.Location) > 512 && aBot.FindInventoryType(class'GusPackII.Sporifier') != None )
					aBot.Weapon = Weapon(aBot.FindInventoryType(class'GusPackII.Sporifier'));
			
				return 1;
			}
		}
			
		return 3;
	}
}

simulated function bool SpotMush(Pawn Other, Pawn Finder)
{
	if ( !Other.bIsPlayer || Other.PlayerReplicationInfo == none || Other.FindInventoryType(class'GusPackII.Spottedmush') != none )
		return false;

	Other.AddInventory(Spawn(class'GusPackII.Spottedmush'));
	
	Broadcastmessage(Other.PlayerReplicationInfo.Playername@"was discovered as mush by"@Finder.PlayerReplicationInfo.PlayerName$"!", true, 'CriticalEvent');
	
	return true;
}

simulated function bool RestartPlayer(Pawn aPlayer)
{
	if ( bMatchEnd )
	{
		EndGame("unknownvictory");
	
		return false;
	}

	if ( aPlayer.PlayerReplicationInfo.Deaths > 0 && bSelected )
		return false;

	aPlayer.PlayerReplicationInfo.Team = 0;

	return Super.RestartPlayer(APlayer);
}

simulated function bool SetUpBot(Bot b)
{
	if ( b == none )
	{
		Warn("ERROR: Added bot not found!");
		
		return true;
	}
	
	b.PlayerReplicationInfo.Team = 0;
	
	return true;
}

simulated function bool AddBot()
{
	local Pawn p;
	local Bot b;

	if ( !Super.AddBot() )
		return false;
	
	for ( p = Level.PawnList; p != none; p = p.nextPawn )
		if ( Bot(p) != none ) b = Bot(p);
		
	return SetUpBot(b);
}

simulated function bool ForceAddBot()
{
	local Pawn p;
	local Bot b;

	if ( !Super.ForceAddBot() )
		return false;
	
	for ( p = Level.PawnList; p != none; p = p.nextPawn )
		if ( Bot(p) != none ) b = Bot(p);
		
	return SetUpBot(b);
}

simulated function Timer()
{
	Super.Timer();

	if ( bRequireReady && (CountDown > 0) )
	{
		bStarted = true;
	
		GoToState('GameStarted');
	}
}

simulated function bool ChangeTeam(Pawn Other, int N) 
{
	if ( !bSelected ) return Super.ChangeTeam(Other, n);
	
	return false;
}

simulated function int CountPlayers()
{
	local int i;
	local Pawn p;
	
	for ( p = Level.PawnList; p != none; p = p.NextPawn )
		if ( p.bIsPlayer )
			i++;
			
	return i;
}

simulated function int MushCount()
{
	local int i;
	local Pawn p;
	
	for ( p = Level.PawnList; p != none; p = p.NextPawn )
		if ( p.bIsPlayer && p.PlayerReplicationInfo != none && p.PlayerReplicationInfo.Team == 1 )
			i++;
			
	return i;
}

auto state Idle {}

auto state GameStarted
{
	function Selected()
	{
		local Pawn p;
		local Sporifier sp;
		local Weapon w;
	
		foreach AllActors(class'Sporifier', sp)
			sp.MushSelected();
	
		for ( p = Level.PawnList; p != none; p = p.NextPawn )
			if ( p.bIsPlayer && p.PlayerReplicationInfo != none )
			{
				// Log("Spawning monitor at:"@p@p.PlayerReplicationInfo.PlayerName);
				Spawn(class'GusPackII.PlayerMonitor', p,, p.Location);
				w = p.Weapon;
				GiveWeapon(p, "GusPackII.MushBeacon");
				p.Weapon = w;
			}
	}

	function SelectTraitor()
	{
		local Pawn p, c;
		
		while ( p == none )
			for ( c = Level.PawnList; c != none; c = c.nextPawn )
				if ( FRand() < 0.08 && c.bIsPlayer && c.PlayerReplicationInfo != none && c.PlayerReplicationInfo.Team == 0 )
					p = c;
					
		p.PlayerReplicationInfo.Team = 1;
		p.AddInventory(Spawn(class'GusPackII.Sporifier'));
		
		// Log(p@"("$p.PlayerReplicationInfo.PlayerName$") is now mush!");
	}
	
Begin:
	Sleep(5.0);
	
	Selected();

	while ( CountPlayers() > MushCount() * 3 )
		SelectTraitor();
	
	Broadcastmessage("Selected this match's mushes!", true, 'CriticalEvent');
	bSelected = true;
}

defaultproperties
{
    bNoMonsters=false
    MapPrefix="DM"
    bCoopWeaponMode=true
    BeaconName="MUSH"
    GameName="Mush Match"
    MaxCommanders=2
    StartUpMessage="Kill any suspiciously mush players to win the match! Don't trust skin colors."
    MaxTeams=2
    MaxAllowedTeams=2
	HUDType=class'GusPackII.MushMatchHUD'
	TeamColor(0)="Humans"
    TeamColor(1)="Mushes"
	ScoreBoardType=class'GusPackII.MushMatchScoreBoard'
	MutatorClass=class'GusPackII.MushMatchMutator'
	GameEndedMessage="win the match!"
}
