//=============================================================================
// RecurseNode.
//=============================================================================
class RecurseNode expands PathNode;

var()	int  	Steps;
var()	float	MaxWalkingPitch;
var()	float	MaxDensity, MinDensity;
var()	float	WalkHeight;
var()	bool	bDebug;

function CheckLog(string s)
{
	if ( bDebug )
		Log(s);
}

function bool SurfaceWalkable(vector HitNormal)
{
	return (abs(Rotator(HitNormal).Pitch) / 65536.0) < MaxWalkingPitch;
}

function bool TraceWalk2(out Vector HitLoct, out Vector HitNorm, vector Start, float WalkDistance, float YawStep, optional int Steps)
{
	local Rotator FinalDir;
	local Vector DirOff;
	local bool b;

	if ( Steps == 0 )
		Steps = 8;
	
	FinalDir = Rotation;
	FinalDir.Yaw += YawStep * (65536 / Steps);

	DirOff = Vector(FinalDir);
	DirOff.X *= WalkDistance;
	DirOff.Y *= WalkDistance;
	DirOff.Z -= WalkHeight * 2 + 32;

	b = Trace(HitLoct, HitNorm, Start + DirOff, Start, false) != None || !FastTrace(Start + DirOff, Start);
		
	CheckLog(Start@"+"@DirOff@"="@Start + DirOff@"o"@FinalDir.Yaw@"^"@Rotator(HitNorm).Pitch@"->"@HitLoct@"=="@b);
	
	return b;
}

function PlopNodes()
{
	local int st;
	local vector L, N;
	local Rotator newRot;
	local RecurseNode RN;
	local NavigationPoint C;

	for ( st = 0; st < Steps; st++ )
	{
		if ( !TraceWalk2(L, N, Location, MaxDensity, st, Steps) )
		{
			CheckLog(Name@"step"@st@"couldn't be traced!");
			continue;
		}

		if ( !SurfaceWalkable(N) )
		{
			CheckLog(Name@"rstep"@st@"would be plopped on a non-walkable surface!");
			continue;
		}

		foreach RadiusActors(class'NavigationPoint', C, MinDensity, L)
		{
			CheckLog(Name@"rstep"@st@"has already a nearby NavigationPoint!");
			break;
		}

		if ( C != None )
			continue;

		L.Z += class'RecurseNode'.default.CollisionHeight + WalkHeight;

		newRot = Rotator(N);
		newRot.Pitch -= 16384;
		RN = Spawn(class'RecurseNode', self, Tag, L, newRot);
		RN.Steps = Steps;
		RN.MaxWalkingPitch = MaxWalkingPitch;
		RN.MaxDensity = MaxDensity;
		RN.WalkHeight = WalkHeight;
		RN.bDebug = bDebug;
		RN.PlopNodes();
	}
}

function PostBeginPlay()
{
	bHidden = !bDebug;
}

function PreBeginPlay()
{
	if ( Owner == None || RecurseNode(Owner) == None )
		PlopNodes();
}

defaultproperties
{
     Steps=12
     MaxWalkingPitch=0.200000
     MaxDensity=256.000000
     MinDensity=192.000000
     WalkHeight=24.000000
     bStatic=False
     bDirectional=True
}
